@@ -9,9 +9,9 @@ from .nexus_classes import NXroot
 
 class File(AbstractContextManager, NXroot):
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, definition=None, **kwargs):
         self._file = h5py.File(*args, **kwargs)
-        NXroot.__init__(self, self._file)
+        NXroot.__init__(self, self._file, definition=definition)
 
     def __enter__(self):
         self._file.__enter__()
