@@ -6,11 +6,23 @@ from copy import copy
 from typing import List, Optional, Union
 import scipp as sc
 from .nxobject import NXobject, Field, ScippIndex, NexusStructureError
-from .nxobject import is_dataset
-from .nxdata import NXdata
+from .nxobject import is_dataset, asarray
+from .nxdata import NXdata, NXdataStrategy
 from .nxevent_data import NXevent_data
 
 
+class NXdetectorStrategy(NXdataStrategy):
+
+    @staticmethod
+    def signal(group):
+        # NXdata uses the 'signal' attribute to define the field name of the signal.
+        # NXdetector uses a "hard-coded" signal name 'data', without specifying the
+        # attribute in the file, so we pass this explicitly to NXdata.
+        # Note the special case of an NXevent_data subgroup named 'data', which we
+        # avoid by checking if 'data' is a dataset.
+        return 'data' if 'data' in group and is_dataset(group._group['data']) else None
+
+
 def group(da: sc.DataArray, groups: sc.Variable) -> sc.DataArray:
     if hasattr(da, 'group'):
         return da.group(groups)
@@ -91,9 +103,7 @@ class _EventField:
                                  stop=id_max.value + 1,
                                  dtype=id_min.dtype)
         else:
-            grouping = self._grouping[select]
-            if not isinstance(grouping, sc.Variable):
-                grouping = sc.scalar(grouping, unit=None, dtype=self._grouping.dtype)
+            grouping = asarray(self._grouping[select])
             if (self._grouping_key in event_data.coords) and sc.identical(
                     grouping, event_data.coords[self._grouping_key]):
                 return event_data
@@ -158,15 +168,9 @@ class NXdetector(NXobject):
                                  **self._event_grouping)
         else:
             signal = None
-        # NXdata uses the 'signal' attribute to define the field name of the signal.
-        # NXdetector uses a "hard-coded" signal name 'data', without specifying the
-        # attribute in the file, so we pass this explicitly to NXdata.
-        # Note the special case of an NXevent_data subgroup named 'data', which we
-        # avoid by checking if 'data' is a dataset.
         return NXdata(
             self._group,
-            signal_name_default='data'
-            if 'data' in self and is_dataset(self._group['data']) else None,
+            strategy=NXdetectorStrategy,
             signal_override=signal,
             skip=self._nxevent_data_fields if self.events is not None else None)
 
