@@ -120,7 +120,7 @@ class NXevent_data(NXobject):
 
         try:
             binned = sc.bins(data=events, dim=_event_dimension, begin=begins, end=ends)
-        except sc.SliceError as e:
+        except IndexError as e:
             raise IndexError(
                 f"Invalid index in NXevent_data at {self.name}/event_index:\n{e}.")
 
