@@ -8,7 +8,7 @@ import warnings
 import datetime
 import dateutil.parser
 import functools
-from typing import overload, List, Union, Any, Dict, Tuple, Protocol, Optional
+from typing import overload, List, Union, Any, Dict, Tuple, Protocol, Optional, Callable
 import numpy as np
 import scipp as sc
 import h5py
@@ -138,28 +138,28 @@ class Field:
     In HDF5 fields are represented as dataset.
     """
 
-    def __init__(self, dataset: H5Dataset, dims=None, is_time=None):
+    def __init__(self, dataset: H5Dataset, *, ancestor, dims=None, is_time=None):
+        self._ancestor = ancestor  # Ususally the parent, but may be grandparent, etc.
         self._dataset = dataset
-        self._shape = list(self._dataset.shape)
+        self._shape = self._dataset.shape
         self._is_time = is_time
         # NeXus treats [] and [1] interchangeably. In general this is ill-defined, but
         # the best we can do appears to be squeezing unless the file provides names for
         # dimensions. The shape property of this class does thus not necessarily return
         # the same as the shape of the underlying dataset.
         if dims is not None:
-            self._dims = dims
+            self._dims = tuple(dims)
             if len(self._dims) < len(self._shape):
                 # The convention here is that the given dimensions apply to the shapes
                 # starting from the left. So we only squeeze dimensions that are after
                 # len(dims).
-                self._shape = self._shape[:len(self._dims)] + [
-                    size for size in self._shape[len(self._dims):] if size != 1
-                ]
+                self._shape = self._shape[:len(self._dims)] + tuple(
+                    size for size in self._shape[len(self._dims):] if size != 1)
         elif (axes := self.attrs.get('axes')) is not None:
-            self._dims = axes.split(',')
+            self._dims = tuple(axes.split(','))
         else:
-            self._shape = [size for size in self._shape if size != 1]
-            self._dims = [f'dim_{i}' for i in range(self.ndim)]
+            self._shape = tuple(size for size in self._shape if size != 1)
+            self._dims = tuple(f'dim_{i}' for i in range(self.ndim))
 
     def __getitem__(self, select) -> Union[Any, sc.Variable]:
         """Load the field as a :py:class:`scipp.Variable` or Python object.
@@ -255,7 +255,7 @@ class Field:
 
     @property
     def parent(self) -> NXobject:
-        return _make(self._dataset.parent)
+        return self._ancestor._make(self._dataset.parent)
 
     @property
     def ndim(self) -> int:
@@ -306,9 +306,35 @@ class NXobject:
     """Base class for all NeXus groups.
     """
 
-    def __init__(self, group: H5Group):
+    def __init__(self,
+                 group: H5Group,
+                 *,
+                 definition: Any = None,
+                 strategy: Optional[Callable] = None):
         self._group = group
+        # TODO can strategies replace child-params?
         self.child_params = {}
+        self._definition = definition
+        if strategy is not None:
+            self._strategy = strategy
+        elif self._definition is not None:
+            self._strategy = self._definition.make_strategy(self)
+        else:
+            self._strategy = self._default_strategy()
+
+    def _default_strategy(self):
+        """
+        Default strategy to use when none given and when the application definition
+        does not provide one. Override in child classes to set a default.
+        """
+        return None
+
+    def _make(self, group) -> NXobject:
+        if (nx_class := Attrs(group.attrs).get('NX_class')) is not None:
+            return _nx_class_registry().get(nx_class,
+                                            NXobject)(group,
+                                                      definition=self._definition)
+        return group  # Return underlying (h5py) group
 
     def _get_child(
             self,
@@ -321,9 +347,12 @@ class NXobject:
             item = self._group[name]
             if is_dataset(item):
                 dims = self._get_field_dims(name) if use_field_dims else None
-                return Field(item, dims=dims, **self.child_params.get(name, {}))
+                return Field(item,
+                             dims=dims,
+                             ancestor=self,
+                             **self.child_params.get(name, {}))
             else:
-                return _make(item)
+                return self._make(item)
         da = self._getitem(name)
         if (t := self.depends_on) is not None:
             if isinstance(da, dict):
@@ -339,8 +368,9 @@ class NXobject:
         children = {}
         select = tuple(select) if isinstance(select, list) else select
         for key in self.keys():
-            if issubclass(type(child := self._get_child(key)), select):
-                children[key] = child
+            if issubclass(type(self._get_child(key)), select):
+                # Get child again via __getitem__ so correct field dims are used.
+                children[key] = self[key]
         return children
 
     @overload
@@ -416,11 +446,14 @@ class NXobject:
 
     @property
     def parent(self) -> NXobject:
-        return _make(self._group.parent)
+        return self._make(self._group.parent)
 
     def _ipython_key_completions_(self) -> List[str]:
         return list(self.keys())
 
+    def __iter__(self):
+        yield from self._group
+
     def keys(self) -> List[str]:
         return self._group.keys()
 
@@ -453,6 +486,8 @@ class NXobject:
         return f'<{type(self).__name__} "{self._group.name}">'
 
     def create_field(self, name: str, data: DimensionedArray, **kwargs) -> Field:
+        if not isinstance(data, sc.Variable):
+            return self._group.create_dataset(name, data=data, **kwargs)
         values = data.values
         if data.dtype == sc.DType.string:
             values = np.array(data.values, dtype=object)
@@ -464,7 +499,7 @@ class NXobject:
             dataset.attrs['units'] = str(data.unit)
         if data.dtype == sc.DType.datetime64:
             dataset.attrs['start'] = str(start.value)
-        return Field(dataset, data.dims)
+        return Field(dataset, dims=data.dims, ancestor=self)
 
     def create_class(self, name: str, nx_class: Union[str, type]) -> NXobject:
         """Create empty HDF5 group with given name and set the NX_class attribute.
@@ -480,7 +515,7 @@ class NXobject:
         group = self._group.create_group(name)
         attr = nx_class if isinstance(nx_class, str) else nx_class.__name__
         group.attrs['NX_class'] = attr
-        return _make(group)
+        return self._make(group)
 
     def __setitem__(self, name: str, value: Union[Field, NXobject, DimensionedArray]):
         """Create a link or a new field."""
@@ -488,6 +523,9 @@ class NXobject:
             self._group[name] = value._dataset
         elif isinstance(value, NXobject):
             self._group[name] = value._group
+        elif hasattr(value, '__write_to_nexus_group__'):
+            group = self.create_class(name, nx_class=value.nx_class)
+            value.__write_to_nexus_group__(group)
         else:
             self.create_field(name, value)
 
@@ -510,7 +548,7 @@ class NXobject:
         # edge cases where creation of Field/NXobject may raise on unrelated children.
         for name, val in self._group.items():
             if not is_dataset(val):
-                nxclasses.append(_make(val).nx_class)
+                nxclasses.append(self._make(val).nx_class)
         for key in set(nxclasses):
             if key is None:
                 continue
@@ -521,12 +559,6 @@ class NXobject:
         return keys
 
 
-def _make(group) -> NXobject:
-    if (nx_class := Attrs(group.attrs).get('NX_class')) is not None:
-        return _nx_class_registry().get(nx_class, NXobject)(group)
-    return group  # Return underlying (h5py) group
-
-
 class NXroot(NXobject):
     """Root of a NeXus file."""
 
